@@ -1,6 +1,3 @@
-/* Declarations for functions provided by the R language */
-
-
 /*
  * There probably won't be more than one C source file using
  * this header, but it's still a good idea to make sure the
@@ -8,13 +5,18 @@
  * otherwise.
 */
 
-#ifndef HAVE_TIKZDEV_H
+#ifndef HAVE_TIKZDEV_H // Begin once-only header
 
 #define HAVE_TIKZDEV_H 
 
+#ifndef DEBUG
+#define DEBUG FALSE
+#endif
+
 /* Use default graphics engine function declarations. */
 #define R_USE_PROTOTYPES 1
 
+/* Declarations for functions provided by the R language */
 #include <R.h>
 #include <Rinternals.h>
 #include <R_ext/GraphicsEngine.h>
@@ -44,6 +46,8 @@ typedef struct{
 	const char *packages;
 	const char *footer;
 	Rboolean polyLine;
+	Rboolean console;
+	Rboolean sanitize;
 } tikzDevDesc;
 
 
@@ -53,12 +57,11 @@ static Rboolean TikZ_Setup(
 		pDevDesc deviceInfo,
 		const char *fileName,
 		double width, double height,
-		const char *bg, const char *fg,
+		const char *bg, const char *fg, double baseSize,
 		Rboolean standAlone, Rboolean bareBones,
 		const char *documentDeclaration,
-		const char *packages, const char *footer );
-
-double dim2dev( double length );
+		const char *packages, const char *footer,
+		Rboolean console, Rboolean sanitize );
 
 static Rboolean TikZ_Open( pDevDesc deviceInfo );
 
@@ -71,7 +74,7 @@ static void TikZ_Clip( double x0, double x1,
 		double y0, double y1, pDevDesc deviceInfo );
 static void TikZ_Size( double *left, double *right,
 		double *bottom, double *top, pDevDesc deviceInfo);
-
+double TikZ_ScaleFont( const pGEcontext plotParams, pDevDesc deviceInfo );
 
 /* Text routines. */
 static void TikZ_MetricInfo( int c, const pGEcontext plotParams,
@@ -94,19 +97,23 @@ static void TikZ_Polyline( int n, double *x, double *y,
 static void TikZ_Polygon( int n, double *x, double *y,
 		pGEcontext plotParams, pDevDesc deviceInfo );
 		
-/*Internal style definition routines*/
-static void StyleDef(Rboolean defineColor, const pGEcontext plotParams, 
-	pDevDesc deviceInfo);
-static void SetColor(int color, Rboolean def, pDevDesc deviceInfo);
-static void SetFill(int color, Rboolean def, pDevDesc deviceInfo);
-static void SetAlpha(int color, Rboolean fill, pDevDesc deviceInfo);
-static void SetLineStyle(int lty, int lwd, pDevDesc deviceInfo);
-static void SetDashPattern(int lty, FILE *outputFile);
-static void SetLineWeight(int lwd, FILE *outputFile);
-static void SetLineJoin(R_GE_linejoin ljoin, double lmitre, pDevDesc deviceInfo);
-static void SetLineEnd(R_GE_linejoin lend, pDevDesc deviceInfo);
-static void SetMitreLimit(double lmitre, FILE *outputFile);
 
+/* Raster routines are only defined for R >= 2.11.0, Graphics Engine >= 6 */
+#if R_GE_version >= 6
+
+static void TikZ_Raster( 
+  unsigned int *raster,
+  int w, int h,
+  double x, double y,
+  double width, double height,
+  double rot,
+  Rboolean interpolate,
+  const pGEcontext plotParams, pDevDesc deviceInfo
+);
+
+static SEXP TikZ_Cap( pDevDesc deviceInfo );
+
+#endif
 
 /* Dummy routines. */
 static void TikZ_Activate( pDevDesc deviceInfo );
@@ -114,7 +121,28 @@ static void TikZ_Deactivate( pDevDesc deviceInfo );
 static Rboolean TikZ_Locator( double *x, double *y, pDevDesc deviceInfo );
 static void TikZ_Mode( int mode, pDevDesc deviceInfo );
 
+/* End R Graphics engin function hooks. */
+
+
+
+/*Internal style definition routines*/
+static void StyleDef(Rboolean defineColor, const pGEcontext plotParams, 
+	pDevDesc deviceInfo);
+static void SetColor(int color, Rboolean def, tikzDevDesc *tikzInfo);
+static void SetFill(int color, Rboolean def, tikzDevDesc *tikzInfo);
+static void SetAlpha(int color, Rboolean fill, tikzDevDesc *tikzInfo);
+static void SetLineStyle(int lty, int lwd, tikzDevDesc *tikzInfo);
+static void SetDashPattern(int lty, tikzDevDesc *tikzInfo);
+static void SetLineWeight(int lwd, tikzDevDesc *tikzInfo);
+static void SetLineJoin(R_GE_linejoin ljoin, double lmitre, tikzDevDesc *tikzInfo);
+static void SetLineEnd(R_GE_linejoin lend, tikzDevDesc *tikzInfo);
+static void SetMitreLimit(double lmitre, tikzDevDesc *tikzInfo);
+
 /* Auxilury routines*/
 void tikzAnnotate(const char **annotation, int *size);
+double dim2dev( double length );
+static void Print_TikZ_Header( tikzDevDesc *tikzInfo );
+void printOutput(tikzDevDesc *tikzInfo, const char *format, ...);
+static char *Sanitize(const char *str);
 
 #endif // End of Once Only header
